﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2014 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System;
using System.Collections.Generic;
using System.Drawing;
using System.Linq;
using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;

namespace HeuristicLab.EvolutionTracking {
  [Item("DirectedGraph", "Generic class representing a directed graph with custom vertices and content")]
  [StorableClass]
  public class DirectedGraph : Item, IDirectedGraph {
    [Storable]
    private readonly List<IVertex> nodes; // for performance reasons, maybe this should be a linked list (fast remove)
    public IEnumerable<IVertex> Nodes {
      get { return nodes; }
    }

    [Storable]
    private readonly Dictionary<string, IVertex> idMap;

    [Storable]
    private readonly Dictionary<object, IVertex> contentMap;
    public DirectedGraph() {
      nodes = new List<IVertex>();
      contentMap = new Dictionary<object, IVertex>();
      idMap = new Dictionary<string, IVertex>();
    }
    [StorableConstructor]
    protected DirectedGraph(bool serializing)
      : base(serializing) {
    }
    protected DirectedGraph(DirectedGraph original, Cloner cloner)
      : base(original, cloner) {
      nodes = new List<IVertex>(original.Nodes);
      contentMap = new Dictionary<object, IVertex>(original.contentMap);
      idMap = new Dictionary<string, IVertex>(original.idMap);
    }
    public override IDeepCloneable Clone(Cloner cloner) {
      return new DirectedGraph(this, cloner);
    }
    public bool Contains(IVertex t) {
      return nodes.Contains(t);
    }

    public bool Contains(object content) {
      return contentMap.ContainsKey(content);
    }

    public IVertex GetVertex(string id) {
      IVertex result;
      idMap.TryGetValue(id, out result);
      return result;
    }

    public IVertex this[object key] {
      get {
        IVertex result;
        contentMap.TryGetValue(key, out result);
        return result;
      }
      set {
        if (contentMap.ContainsKey(key)) {
          idMap.Remove(contentMap[key].Id);
        }
        contentMap[key] = value;
        idMap[value.Id] = value;
      }
    }
    public virtual bool Any(Func<IVertex, bool> predicate) {
      return nodes.Any(predicate);
    }
    public virtual bool IsEmpty {
      get { return !nodes.Any(); }
    }
    public virtual void Clear() {
      nodes.Clear();
      contentMap.Clear();
      idMap.Clear();
    }
    public virtual void AddVertex(IVertex vertex) {
      if (contentMap.ContainsKey(vertex.Content)) {
        throw new Exception("Content already exists in the graph.");
      }
      contentMap[vertex.Content] = vertex;
      idMap[vertex.Id] = vertex;
      nodes.Add(vertex);
    }

    public virtual void RemoveVertex(IVertex vertex) {
      nodes.Remove(vertex);
    }
    public override Image ItemImage {
      get { return Common.Resources.VSImageLibrary.Graph; }
    }
  }
}
